@@ -25,10 +25,12 @@ struct libinput_category {
 	enum libinput_config_tap_button_map tap_button_map;
 	int tap_and_drag;               /* -1 or libinput_config_drag_state */
 	int drag_lock;                  /* -1 or libinput_config_drag_lock_state */
+	int three_finger_drag;          /* -1 or libinput_config_3fg_drag_state */
 	int accel_profile;              /* -1 or libinput_config_accel_profile */
 	int middle_emu;                 /* -1 or libinput_config_middle_emulation_state */
 	int dwt;                        /* -1 or libinput_config_dwt_state */
 	int click_method;               /* -1 or libinput_config_click_method */
+	int scroll_method;              /* -1 or libinput_config_scroll_method */
 	int send_events_mode;           /* -1 or libinput_config_send_events_mode */
 	bool have_calibration_matrix;
 	double scroll_factor;
@@ -36,6 +38,7 @@ struct libinput_category {
 };
 
 enum lab_libinput_device_type get_device_type(const char *s);
+const char *libinput_device_type_name(enum lab_libinput_device_type type);
 struct libinput_category *libinput_category_create(void);
 struct libinput_category *libinput_category_get_default(void);
 
