@@ -1,14 +1,10 @@
 // SPDX-License-Identifier: GPL-2.0-only
-#include <cairo.h>
-#include <drm_fourcc.h>
-#include <pango/pangocairo.h>
-#include <wlr/render/wlr_renderer.h>
-#include <wlr/util/box.h>
-#include <wlr/util/log.h>
 #include "common/font.h"
+#include <cairo.h>
+#include <pango/pangocairo.h>
+#include <wlr/util/log.h>
 #include "common/graphic-helpers.h"
 #include "common/string-helpers.h"
-#include "labwc.h"
 #include "buffer.h"
 
 PangoFontDescription *
@@ -26,7 +22,7 @@ static PangoRectangle
 font_extents(struct font *font, const char *string)
 {
 	PangoRectangle rect = { 0 };
-	if (!string) {
+	if (string_null_or_empty(string)) {
 		return rect;
 	}
 	cairo_surface_t *surface;
@@ -47,10 +43,6 @@ font_extents(struct font *font, const char *string)
 	pango_layout_get_extents(layout, NULL, &rect);
 	pango_extents_to_pixels(&rect, NULL);
 
-	/* we put a 2 px edge on each side - because Openbox does it :) */
-	/* TODO: remove the 4 pixel addition and always do the padding by the caller */
-	rect.width += 4;
-
 	cairo_destroy(c);
 	cairo_surface_destroy(surface);
 	pango_font_description_free(desc);
@@ -86,15 +78,18 @@ font_get_buffer_size(int max_width, const char *text, struct font *font,
 
 void
 font_buffer_create(struct lab_data_buffer **buffer, int max_width,
-	const char *text, struct font *font, const float *color,
-	const float *bg_color, double scale)
+	int height, const char *text, struct font *font, const float *color,
+	cairo_pattern_t *bg_pattern, double scale)
 {
 	if (string_null_or_empty(text)) {
 		return;
 	}
 
-	int width, height;
-	font_get_buffer_size(max_width, text, font, &width, &height);
+	int width, computed_height;
+	font_get_buffer_size(max_width, text, font, &width, &computed_height);
+	if (height <= 0) {
+		height = computed_height;
+	}
 
 	*buffer = buffer_create_cairo(width, height, scale);
 	if (!*buffer) {
@@ -114,18 +109,16 @@ font_buffer_create(struct lab_data_buffer **buffer, int max_width,
 	 * buffer unfilled (completely transparent) since the background
 	 * is already rendered by the scene element underneath. In this
 	 * case we have to disable subpixel rendering.
-	 *
-	 * Note: the 0.999 cutoff was chosen to be greater than 254/255
-	 * (about 0.996) but leave some margin for rounding errors.
 	 */
-	bool opaque_bg = (bg_color[3] > 0.999f);
+	bool opaque_bg = is_pattern_opaque(bg_pattern);
 	if (opaque_bg) {
-		set_cairo_color(cairo, bg_color);
+		cairo_set_source(cairo, bg_pattern);
 		cairo_paint(cairo);
 	}
 
 	set_cairo_color(cairo, color);
-	cairo_move_to(cairo, 0, 0);
+	/* center vertically if height was explicitly specified */
+	cairo_move_to(cairo, 0, (height - computed_height) / 2);
 
 	PangoLayout *layout = pango_cairo_create_layout(cairo);
 	pango_context_set_round_glyph_positions(pango_layout_get_context(layout), false);
