@@ -1,30 +1,11 @@
 // SPDX-License-Identifier: GPL-2.0-only
 /* view-impl-common.c: common code for shell view->impl functions */
-#include <stdio.h>
-#include <strings.h>
-#include "common/list.h"
-#include "foreign-toplevel.h"
+#include "view-impl-common.h"
+#include "foreign-toplevel/foreign.h"
 #include "labwc.h"
 #include "view.h"
-#include "view-impl-common.h"
 #include "window-rules.h"
 
-void
-view_impl_move_to_front(struct view *view)
-{
-	wl_list_remove(&view->link);
-	wl_list_insert(&view->server->views, &view->link);
-	wlr_scene_node_raise_to_top(&view->scene_tree->node);
-}
-
-void
-view_impl_move_to_back(struct view *view)
-{
-	wl_list_remove(&view->link);
-	wl_list_append(&view->server->views, &view->link);
-	wlr_scene_node_lower_to_bottom(&view->scene_tree->node);
-}
-
 void
 view_impl_map(struct view *view)
 {
@@ -63,16 +44,21 @@ void
 view_impl_unmap(struct view *view)
 {
 	struct server *server = view->server;
-	if (view == server->active_view) {
+	/*
+	 * When exiting an xwayland application with multiple views
+	 * mapped, a race condition can occur: after the topmost view
+	 * is unmapped, the next view under it is offered focus, but is
+	 * also unmapped before accepting focus (so server->active_view
+	 * remains NULL). To avoid being left with no active view at
+	 * all, check for that case also.
+	 */
+	if (view == server->active_view || !server->active_view) {
 		desktop_focus_topmost_view(server);
 	}
-	if (view == server->last_raised_view) {
-		server->last_raised_view = NULL;
-	}
 }
 
 static bool
-resizing_edge(struct view *view, uint32_t edge)
+resizing_edge(struct view *view, enum lab_edge edge)
 {
 	struct server *server = view->server;
 	return server->input_mode == LAB_INPUT_STATE_RESIZE
@@ -99,7 +85,7 @@ view_impl_apply_geometry(struct view *view, int w, int h)
 	 * reliable on its own. The combination of the two methods
 	 * should catch 99% of resize cases that we care about.
 	 */
-	bool resizing_left_edge = resizing_edge(view, WLR_EDGE_LEFT);
+	bool resizing_left_edge = resizing_edge(view, LAB_EDGE_LEFT);
 	if (resizing_left_edge || (current->x != pending->x
 			&& current->x + current->width ==
 			pending->x + pending->width)) {
@@ -109,7 +95,7 @@ view_impl_apply_geometry(struct view *view, int w, int h)
 	}
 
 	/* Anchor bottom edge if resizing via top edge */
-	bool resizing_top_edge = resizing_edge(view, WLR_EDGE_TOP);
+	bool resizing_top_edge = resizing_edge(view, LAB_EDGE_TOP);
 	if (resizing_top_edge || (current->y != pending->y
 			&& current->y + current->height ==
 			pending->y + pending->height)) {
